package picassotransformwrapper;

import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.util.DisplayMetrics;
import android.view.ViewGroup;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Pixel;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.ViewWrapper;
//import anywheresoftware.b4a.objects.TextViewWrapper;
import anywheresoftware.b4a.BA.ActivityObject;

import anywheresoftware.b4a.BA.Events;

import android.content.Context;
import android.graphics.Color;
import android.graphics.PointF;
import android.widget.ImageView;
import android.widget.TextView;
import com.squareup.picasso.Picasso;
import java.util.List;

import jp.wasabeef.picasso.transformations.RoundedCornersTransformation;


//@ActivityObject
@Author("Github: Daichi Furiya, Wrapped by: Johan Schoeman")
@Version(1.00f)
@ShortName("RoundedCornersTransformation")
@DependsOn(values={"picasso-2.5.2"})
public class RoundedCornersWrapper {


	private BA ba;
	private RoundedCornersTransformation rct;

	
 	public void Initialize(BA ba) {
         this.ba = ba;
	}
 
    private Bitmap bm = null;
    public void setImageBitmap(Bitmap bm) {
	  this.bm = bm;
	}  
 
    private int radius = 40;
    public void setCornerRadius (int radius) {
		this.radius = radius;
	}
	
    private int margin = 15;
    public void setImageMargin (int margin) {
		this.margin = margin;
	}	
	
	private int mCornerType = 0;
	public void setCornerType(int mCornerType) {
		this.mCornerType = mCornerType;
	}


    public final int CornerType_ALL = 0;
	public final int CornerType_TOP_LEFT = 1;
	public final int CornerType_TOP_RIGHT = 2;
	public final int CornerType_BOTTOM_LEFT = 3;
	public final int CornerType_BOTTOM_RIGHT = 4;
	public final int CornerType_TOP = 5;
	public final int CornerType_BOTTOM = 6;
	public final int CornerType_LEFT = 7;
	public final int CornerType_RIGHT = 8;
	public final int CornerType_OTHER_TOP_LEFT = 9;
	public final int CornerType_OTHER_TOP_RIGHT = 10;
	public final int CornerType_OTHER_BOTTOM_LEFT = 11;
	public final int CornerType_OTHER_BOTTOM_RIGHT = 12;
	public final int CornerType_DIAGONAL_FROM_TOP_LEFT = 13;
	public final int CornerType_DIAGONAL_FROM_TOP_RIGHT = 14;
	
	public Object applyRoundedCornersTransformation() {
		switch (mCornerType) {
		  case CornerType_ALL:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.ALL);
			break;
		  case CornerType_TOP_LEFT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.TOP_LEFT);
			break;
		  case CornerType_TOP_RIGHT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.TOP_RIGHT);
			break;
		  case CornerType_BOTTOM_LEFT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.BOTTOM_LEFT);
			break;
		  case CornerType_BOTTOM_RIGHT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.BOTTOM_RIGHT);
			break;
		  case CornerType_TOP:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.TOP);
			break;
		  case CornerType_BOTTOM:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.BOTTOM);
			break;
		  case CornerType_LEFT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.LEFT);
			break;
		  case CornerType_RIGHT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.RIGHT);
			break;
		  case CornerType_OTHER_TOP_LEFT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.OTHER_TOP_LEFT);
			break;
		  case CornerType_OTHER_TOP_RIGHT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.OTHER_TOP_RIGHT);
			break;
		  case CornerType_OTHER_BOTTOM_LEFT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.OTHER_BOTTOM_LEFT);
			break;
		  case CornerType_OTHER_BOTTOM_RIGHT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.OTHER_BOTTOM_RIGHT);
			break;
		  case CornerType_DIAGONAL_FROM_TOP_LEFT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.DIAGONAL_FROM_TOP_LEFT);
			break;
		  case CornerType_DIAGONAL_FROM_TOP_RIGHT:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.DIAGONAL_FROM_TOP_RIGHT); 
			break;
		  default:
			rct = new RoundedCornersTransformation(radius, margin, RoundedCornersTransformation.CornerType.ALL);
			break;
		}		
		
	    return rct.transform(bm); 	
		
		
    }

   
}
